@@ -1,10 +1,15 @@
-/* $Id: ip4r.c,v 1.5 2006/10/08 17:10:02 andrewsn Exp $ */
+/* $Id: ip4r.c,v 1.8 2008/01/28 23:18:35 andrewsn Exp $ */
 /*
   New type 'ip4' used to represent a single IPv4 address efficiently
 
   New type 'ip4r' used to represent a range of IPv4 addresses, along
   with support for GiST and rtree indexing of the type.
 
+  V1.03: fix the inet conversions which were not handling short varlenas
+  correctly on 8.3
+
+  V1.02: updates for 8.3
+
   V1.01: updates for 8.2
 
   V0.1: updates for 8.1. add some documentation and new functions.
@@ -392,6 +397,32 @@ bool ip4r_extends_right_of_internal(IP4R *left, IP4R *right)
 #error "Unknown or unsupported postgresql version"
 #endif
 
+/* 8.3 changes the varlena header (to support "short" varlenas without
+ * needing a full 32-bit length field) and changes the varlena macros
+ * to support this. Keep the new interface (which is cleaner than the
+ * old one anyway) and emulate it for older versions.
+ *
+ * But for the "inet" type, unlike all other types, 8.3 does not convert
+ * the packed short format back to the unpacked format in the normal
+ * parameter macros. So we need additional macros to hide that change.
+ * We implicitly assume here that none of the fields of 'inet_struct'
+ * require any alignment stricter than byte (which is true at the moment,
+ * but it's anyone's guess whether it will stay true, given how often it
+ * seems to get changed).
+ */
+
+#if IP4R_PGVER >= 8003000
+
+#define INET_STRUCT_DATA(is_) ((inet_struct *)VARDATA_ANY(is_))
+
+#else /* IP4R_PGVER < 8003000 */
+
+#define SET_VARSIZE(var_,val_) VARATT_SIZEP(var_) = (val_)
+
+#define INET_STRUCT_DATA(is_) ((inet_struct *)VARDATA(is_))
+
+#endif
+
 /* PG_MODULE_MAGIC was introduced in 8.2. */
 
 #if IP4R_PGVER >= 8002000
@@ -548,7 +579,7 @@ text *
 make_text(char *str, int len)
 {
     text *ret = (text *) palloc(len + VARHDRSZ);
-    VARATT_SIZEP(ret) = len + VARHDRSZ;
+    SET_VARSIZE(ret, len + VARHDRSZ);
     if (str)
 	memcpy(VARDATA(ret), str, len);
     else
@@ -560,8 +591,8 @@ static inline
 void
 set_text_len(text *txt, int len)
 {
-    if ((len + VARHDRSZ) < VARATT_SIZEP(txt))
-	VARATT_SIZEP(txt) = len + VARHDRSZ;
+    if ((len + VARHDRSZ) < VARSIZE(txt))
+      SET_VARSIZE(txt, len + VARHDRSZ);
 }
 
 static inline
@@ -668,7 +699,7 @@ Datum
 ip4_cast_from_inet(PG_FUNCTION_ARGS)
 {
     inet *inetptr = PG_GETARG_INET_P(0);
-    inet_struct *in = ((inet_struct *)VARDATA(inetptr));
+    inet_struct *in = INET_STRUCT_DATA(inetptr);
 
     if (in->family == PGSQL_AF_INET)
     {
@@ -691,7 +722,7 @@ ip4_cast_to_cidr(PG_FUNCTION_ARGS)
     inet *res = palloc0(VARHDRSZ + sizeof(inet_struct));
     inet_struct *in;
 
-    VARATT_SIZEP(res) = VARHDRSZ + offsetof(inet_struct, INET_IPADDR) + 4;
+    SET_VARSIZE(res, VARHDRSZ + offsetof(inet_struct, INET_IPADDR) + 4);
 
     in = ((inet_struct *)VARDATA(res));
     INET_INIT_CIDR(in);
@@ -1033,7 +1064,7 @@ Datum
 ip4r_cast_from_cidr(PG_FUNCTION_ARGS)
 {
     inet *inetptr = PG_GETARG_INET_P(0);
-    inet_struct *in = ((inet_struct *)VARDATA(inetptr));
+    inet_struct *in = INET_STRUCT_DATA(inetptr);
 
     if (INET_IS_CIDR(in) && in->family == PGSQL_AF_INET)
     {
@@ -1068,7 +1099,7 @@ ip4r_cast_to_cidr(PG_FUNCTION_ARGS)
 	PG_RETURN_NULL();
 
     res = palloc0(VARHDRSZ + sizeof(inet_struct));
-    VARATT_SIZEP(res) = VARHDRSZ + offsetof(inet_struct, INET_IPADDR) + 4;
+    SET_VARSIZE(res, VARHDRSZ + offsetof(inet_struct, INET_IPADDR) + 4);
 
     in = ((inet_struct *)VARDATA(res));
     INET_INIT_CIDR(in);
